﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace PovrchAObjemKoule
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// Statická třída obsahující deklaraci všech potřebných konstant
        /// </summary>
        static class Constants
        {
            public const string ZadPolomer = "Zadejte poloměr";
        }


        public MainWindow()
        {
            InitializeComponent();
            TxtPolomer.Text = Constants.ZadPolomer;
        }

        private void BtnKonec_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void TxtPolomer_GotFocus(object sender, RoutedEventArgs e)
        {
            if (TxtPolomer.Text == Constants.ZadPolomer)
            {
                TxtPolomer.Text = string.Empty;
            }
        }

        private void TxtPolomer_LostFocus(object sender, RoutedEventArgs e)
        {
            if (TxtPolomer.Text == string.Empty)
            {
                // vyplnění políčka pro poloměr textem proměnné ZadPolomer
                TxtPolomer.Text = Constants.ZadPolomer;
                // vymazání výsledků předchozích výpočtů
                TxtPovrch.Text = "";
                TxtObjem.Text = "";
            }
        }

        private void BtnMetoda_Click(object sender, RoutedEventArgs e)
        {
            // Deklarace lokálních proměnných: s - povrch koule, v - objem koule
            double s, v;

            if (double.TryParse(TxtPolomer.Text, out double r))
            {
                // výpočet povrchu pomocí konstanty PI
                s = 4 * Math.PI * r * r;
                // výpočet objemu s pomocí metody POW
                v = ((double)4 / 3) * Math.PI * Math.Pow(r, 3);

                // vypsání s, v do textboxů
                TxtPovrch.Text = s.ToString();
                TxtObjem.Text = v.ToString();

            }
            // když se nepovede převod textu z GUI na číslo, tak ukážeme warning
            else
            {
                MessageBox.Show("Zadejte číslo ve správném formátu!", "Chyba", MessageBoxButton.OK, MessageBoxImage.Warning);
                TxtPolomer.Focus();
            }



        }

        #region Výpočty

        /// <summary>
        /// Metoda s návratem pro výpočet povrchu koule
        /// </summary>
        /// <param name="r"> poloměr koule - vstupní parametr metody</param>
        /// <returns>výstup je povrch koule (double)</returns>
        private double PovrchKouleMetodaSNavratem(double r)
        {
            return (4 * Math.PI * Math.Pow(r, 2));
        }

        /// <summary>
        /// Metoda s návratem pro výpočet objemu koule
        /// </summary>
        /// <param name="r"> poloměr koule - vstupní parametr metody</param>
        /// <returns>výstup je objem koule (double)</returns>
        private double ObjemKouleMetodaSNavratem(double r)
        {
            return (4d / 3 * Math.PI * Math.Pow(r, 3));
        }

        /// <summary>
        /// Metoda orientovaná na výraz pro výpočet povrchu koule
        /// </summary>
        /// <param name="r"> poloměr koule </param>
        /// <returns> objem koule </returns>
        private double PovrchKouleMetodaVyraz(double r) => 4 * Math.PI * r * r;

        /// <summary>
        /// Metoda orientovaná na výraz pro výpočet objemu koule
        /// </summary>
        /// <param name="r"> poloměr koule </param>
        /// <returns> objem koule</returns>
        private double ObjemKouleMetodaVyraz(double r) => (4d / 3) * Math.PI * Math.Pow(r, 3);

        /// <summary>
        /// Metoda s výstupními parametry "out"
        /// </summary>
        /// <param name="r"> poloměr koule</param>
        /// <param name="s"> povrch koule </param>
        /// <param name="v"> objem koule </param>
        private void PovrchObjemKouleVystupniParametry(double r, out double s, out double v)
        {
            s = 4 * Math.PI * r * r;
            v = ((double)4 / 3) * Math.PI * Math.Pow(r, 3);
        }
         /// <summary>
         /// Metoda vracející 2 výstupy (pomocí tuple)
         /// </summary>
         /// <param name="r"> poloměr koule </param>
         /// <returns> povrch a objem jako tuple</returns>
        private (double, double) PovrchObjemKouleMetodaDvaNavraty(double r)
        {
            double s, v;
            s = 4 * Math.PI * r * r;
            v = ((double)4 / 3) * Math.PI * Math.Pow(r, 3);
            return (s, v);
        }

        /// <summary>
        /// Metoda bez návratu - předání parametrů odkazem (referencí)
        /// </summary>
        /// <param name="r"> poloměr koule </param>
        /// <param name="s"> povrch koule </param>
        /// <param name="v"> objem koule </param>
        private void PovrchObjemKoulePredaniOdkazem(double r, ref double s, ref double v)
        {
            s = 4 * Math.PI * r * r;
            v = ((double)4 / 3) * Math.PI * Math.Pow(r, 3);
        }

        #endregion

        private void BtnMetodaNavrat_Click(object sender, RoutedEventArgs e)
        {
            // parsuju text na číslo
            if (double.TryParse(TxtPolomer.Text, out double r))
            {
                //když se parsování povede, zavolám metody a předám jim parametr r
                TxtPovrch.Text = PovrchKouleMetodaSNavratem(r).ToString();
                TxtObjem.Text = ObjemKouleMetodaSNavratem(r).ToString();
            }
            // když se nepovede převod textu z GUI na číslo, tak ukážeme warning
            else
            {
                MessageBox.Show("Zadejte číslo ve správném formátu!", "Chyba", MessageBoxButton.OK, MessageBoxImage.Warning);
                TxtPolomer.Focus();
            }
        }

        private void BtnMetodaVyraz_Click(object sender, RoutedEventArgs e)
        {
            if (double.TryParse(TxtPolomer.Text, out double r))
            {
                //volám metody pro výpočet objemu a povrchu koule zaměřené na výraz
                TxtPovrch.Text = string.Format("{0:N2}", PovrchKouleMetodaVyraz(r));
                TxtObjem.Text = string.Format("{0:N2}", ObjemKouleMetodaVyraz(r));
            }
            else
            {
                MessageBox.Show("Zadejte číslo ve správném formátu!", "Chyba", MessageBoxButton.OK, MessageBoxImage.Warning);
                TxtPolomer.Focus();
            }

        }

        private void BtnMetodaParametry_Click(object sender, RoutedEventArgs e)
        {
            if (double.TryParse(TxtPolomer.Text, out double r))
            {
                PovrchObjemKouleVystupniParametry(r, out double s, out double v);
                TxtObjem.Text = v.ToString();
                TxtPovrch.Text = s.ToString();
            }
            else
            {
                MessageBox.Show("Zadejte číslo ve správném formátu!", "Chyba", MessageBoxButton.OK, MessageBoxImage.Warning);
                TxtPolomer.Focus();
            }

        }

        private void BtnMetoda2Navraty_Click(object sender, RoutedEventArgs e)
        {
            double s, v;

            if (double.TryParse(TxtPolomer.Text, out double r))
            {
                (s, v) = PovrchObjemKouleMetodaDvaNavraty(r);
                TxtPovrch.Text = string.Format("{0:N2}", s);
                TxtObjem.Text = string.Format("{0:N2}", v);
            }
            else
            {
                MessageBox.Show("Zadejte číslo ve správném formátu!", "Chyba", MessageBoxButton.OK, MessageBoxImage.Warning);
                TxtPolomer.Focus();
            }
        }

        private void BtnMetodaReference_Click(object sender, RoutedEventArgs e)
        {
            double s, v;

            if (double.TryParse(TxtPolomer.Text, out double r))
            {
                s = v = 0;
                PovrchObjemKoulePredaniOdkazem(r, ref s, ref v);
                TxtPovrch.Text = string.Format("{0:N2}", s);
                TxtObjem.Text = string.Format("{0:N2}", v);
            }
            else
            {
                MessageBox.Show("Zadejte číslo ve správném formátu!", "Chyba", MessageBoxButton.OK, MessageBoxImage.Warning);
                TxtPolomer.Focus();
            }
        }
    }
}
